<?php

namespace Drupal\tmgmt_contentapi\Plugin\tmgmt\Translator;

use Drupal\Core\Render\Markup;
use Drupal\node\Entity\Node;
use Drupal\file\Entity\File;
use Drupal\Component\Utility\Xss;
use Drupal\Core\Entity\Entity;
use Drupal\tmgmt\Entity\Job;
use Drupal\tmgmt\Entity\JobItem;
use Drupal\tmgmt\JobInterface;
use Drupal\tmgmt\Translator\TranslatableResult;
use Drupal\tmgmt\TranslatorInterface;
use Drupal\tmgmt\TranslatorPluginBase;
use Drupal\tmgmt_contentapi\Plugin\tmgmt_contentapi\Format\Xliff;
use Drupal\tmgmt_contentapi\Util\GeneralHelper;
use ZipArchive;
use Drupal\tmgmt_contentapi\Swagger\Client\Model\ProviderId;
use Drupal\tmgmt_contentapi\Swagger\Client\Model\CreateRequestUpdateTM;
use Drupal\tmgmt_contentapi\Swagger\Client\Model\CreateRequestFile;
use Drupal\tmgmt_contentapi\Swagger\Client\Model\CreateToken;
use Drupal\tmgmt_contentapi\Swagger\Client\Api\TokenApi;
use Drupal\tmgmt_contentapi\Swagger\Client\Api\TranslationMemoryApi;
use Drupal\tmgmt_contentapi\Swagger\Client\Api\JobApi;
use Drupal\tmgmt_contentapi\Swagger\Client\Api\RequestApi;
use Drupal\tmgmt_contentapi\Swagger\Client\Api\SourceFileApi;
use Drupal\tmgmt_contentapi\Util\ConentApiHelper;
use Drupal\tmgmt\ContinuousTranslatorInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Datetime\DrupalDateTime;

/**
 * Content API Translator.
 *
 * @TranslatorPlugin(
 *   id = "contentapi",
 *   label = @Translation("Lionbridge Content API Connector"),
 *   logo = "icons/lionbridge.png",
 *   description = @Translation("Provider to send content to Lionbridge"),
 *   ui = "Drupal\tmgmt_contentapi\ContentApiTranslatorUI"
 * )
 */
class ContentApiTranslator extends TranslatorPluginBase {

  /**
   * {@inheritdoc}
   */
  #[\ReturnTypeWillChange]
    public function checkTranslatable(TranslatorInterface $translator, JobInterface $job) {
    // Anything can be exported.
    return TranslatableResult::yes();
    }

    /**
     * {@inheritdoc}
     */
    #[\ReturnTypeWillChange]
    public function abortTranslation(JobInterface $job) {
      // Assume that we can abort a translation job at any time.

    }

    /**
     * {@inheritdoc}
     */
    #[\ReturnTypeWillChange]
    public function requestTranslation(JobInterface $job) {

      $jobtranslator = $job->getTranslator();
      $jobtranslatorId = $jobtranslator->getPluginId();
      // Modify operations only for contentapi translator.
      if ($jobtranslatorId == 'contentapi') {
        $source = $this->requestTranslationSource($job);
      }
    }

    /**
     * Request translation source.
     */
    private function requestTranslationSource(JobInterface $job) {
      // Defined here as they will be used in catch block.
      $createdcpjob = NULL;
      $token = NULL;
      $zipPath = NULL;
      $ziparchive = NULL;
      $jobapi = NULL;
      $allfilespath = NULL;
      try {
        if ($job->getSetting('capi-settings')["task"] == "trans") {
          // Submit job.
          \Drupal::service('tmgmt_contentapi.create_job')->submitJobToContentApi($job);
        }
        else {
          // Export files: gernerate paths and other variables.
          $exporter = new Xliff();
          $filesystem = \Drupal::service('file_system');
          $joblabel = GeneralHelper::getJobLabelNoSpeChars($job);
          $dirnameallfiles = $joblabel . '_tmupdate' . $job->id() . "_" . $job->getRemoteSourceLanguage() . "_" . $job->getRemoteTargetLanguage();
          $zipName = 'zip_job_' . $dirnameallfiles . '.zip';
          $allfilespath = $job->getSetting('scheme') . '://tmgmt_contentapi/LioxSentFiles/' . $dirnameallfiles;
          $zipPath = $allfilespath . "/" . $zipName;
          $filearraytodelte = [];
          $filearrayexportedfiles = [];
          $transferfiles = [];
          // Create folder where all exported files will be stored.
          if (\Drupal::service('file_system')->prepareDirectory($allfilespath, FileSystemInterface::CREATE_DIRECTORY)) {
            // Export each item of the job in same file.
            if ($oneexportfile) {
              $labelname = $joblabel;
              $name = $labelname . "_" . $job->id() . "_all_" . $job->getRemoteSourceLanguage() . '_' . $job->getRemoteTargetLanguage() . '.xlf';
              $jobpath = $allfilespath . "/" . $name;
              $file = \Drupal::service('file.repository')->writeData($exporter->export($job), $jobpath, FileSystemInterface::EXISTS_REPLACE);
              $filearraytodelte[] = $file;
              $filearrayexportedfiles['all'] = $file;
            }
            else {
              // Export each item of the job in separate file.
              foreach ($job->getItems() as $item) {
                $labelname = GeneralHelper::getStringNoSpeChars($item->label());
                $name = $labelname . "_" . $job->id() . "_" . $item->id() . "_" . $job->getRemoteSourceLanguage() . '_' . $job->getRemoteTargetLanguage() . '.xlf';
                $itempath = $allfilespath . "/" . $name;
                $file = \Drupal::service('file.repository')->writeData($exporter->exportItem($item), $itempath, FileSystemInterface::EXISTS_REPLACE);
                $filearraytodelte[] = $file;
                $filearrayexportedfiles[$item->id()] = $file;
              }
            }
            // Zip the exported files.
            $ziparchive = new \ZipArchive();
            $openresult = $ziparchive->open($filesystem->realpath($zipPath), ZIPARCHIVE::CREATE | ZIPARCHIVE::OVERWRITE);
            $zipcloseresult = FALSE;
            if ($openresult) {
              foreach ($filearrayexportedfiles as $tempfile) {
                $ziparchive->addFile($filesystem->realpath($tempfile->getFileUri()), $tempfile->getFilename());
              }
              $zipcloseresult = $ziparchive->close();
              if ($zipcloseresult) {
                $zipfileobj = GeneralHelper::createFileObject($zipPath);
                \Drupal::service('file.usage')->add($zipfileobj, 'tmgmt_contentapi', 'tmgmt_job', $job->id());
              }
            }
            else {

            }

            $transferfiles = $filearrayexportedfiles;

            // Create Job in Content API.
            $contentapibundle = [];
            $jobrequest = ConentApiHelper::genrateJobRequst($job);
            $jobapi = new JobApi();
            $createdcpjob = NULL;
            try {
              $createdcpjob = $jobapi->jobsPost($token, $jobrequest);
            }
            catch (\Exception $e) {
              $msg = $e->getMessage();
              if (strlen($msg) > 200) {
                $msg = substr($msg, 0, 200);
              }
              $job->addMessage($msg, [], 'error');
              return;
            }
            // Upload files from transferfiles array to CPA.
            $fileapi = new SourceFileApi();
            $transrequestapi = new RequestApi();
            // Array to store temporally translation requst and associated files.
            $contentapitrrequstfiles = [];
            foreach ($transferfiles as $id => $tmpfile) {
              $data = [];
              $data["job_id"] = $createdcpjob->getJobId();
              $data["filename"] = $tmpfile->getFilename();
              $data["filetype"] = $tmpfile->getMimeType();
              $stmrg = \Drupal::service('stream_wrapper_manager')->getViaUri($tmpfile->getFileUri());
              $extpath = $stmrg->realpath();
              $filrequst = new \SplFileObject($extpath);
              $contentapitmpfile = $fileapi->jobsJobIdUploadPost($token, $data["job_id"], $data["filename"], $data["filetype"], $filrequst);
              $uploadedFile = $fileapi->uploadFile($contentapitmpfile, $extpath);

              $request = new CreateRequestFile([
                "request_name" => $job->id() . "_" . $id . "_" . $job->getRemoteSourceLanguage() . "-" . $job->getRemoteTargetLanguage(),
                "source_native_id" => $job->id() . "_" . $id,
                "source_native_language_code" => $job->getRemoteSourceLanguage(),
                "target_native_language_codes" => [$job->getRemoteTargetLanguage()],
                "file_id" => $contentapitmpfile['fms_file_id'],
              ]);
              $contentapitmprequst = $transrequestapi->jobsJobIdRequestsAddfilePost($token, $data["job_id"], $request);
              $contentapibundle[] = $createdcpjob;
            }
            try {
              $ref = $this->requestTranslationRef($job, $createdcpjob);
            }
            catch (Exception $exc) {

            }
            // Submit content api job.
            $pr = $job->getSetting("capi-settings")["provider"];
            $jid = $createdcpjob->getJobId();
            $prmodel = new ProviderId(['provider_id' => $pr]);
            $jobsubmitresult = $jobapi->jobsJobIdSubmitPut($token, $jid, $prmodel);
            if ($jobsubmitresult) {
              // TODO.
              GeneralHelper::addCpaSettingsToJob($job, serialize($contentapibundle));
              $messageTopass .= '<a href="' . \Drupal::service('file_url_generator')->generateAbsoluteString($zipPath) . '">' . Xss::filter(GeneralHelper::getJobLabelNoSpeChars($job)) . '</a>';
              \Drupal::messenger()->addMessage(Markup::create($messageTopass));
              // Set job to finished.
              $job->setState(Job::STATE_FINISHED);
              foreach ($job->getItems() as $item) {
                $item->setState(JobItem::STATE_ACCEPTED);

              }
              $job->submitted("TM update job sent to provider!");

              // Delete files after sending to Provider.
              foreach ($filearraytodelte as $tempfile) {
                \Drupal::service('file_system')->delete($tempfile->getFileUri());
              }
            }
          }
          else {
            throw new \Exception("Could not create directory for export: " . $allfilespath);
          }
        }
      }
      catch (\Exception $exception) {
        // If exception occurs, clean up everything: delete exported files, cancel job in CA if any.
        foreach ($filearraytodelte as $tempfile) {
          if (file_exists($tempfile->getFileUri())) {
            \Drupal::service('file_system')->delete($tempfile->getFileUri());
          }
        }
        if (!file_exists($zipPath) && $ziparchive != NULL) {
          $ziparchive->close();
        }
        $job->setState(Job::STATE_UNPROCESSED);
        try {
          if (isset($createdcpjob)) {
            // TODO: check why result is NULL, only at certain state job can be deleted? But seems to work!!
            $deletresult = $jobapi->jobsJobIdDelete($token, $createdcpjob->getJobId());
          }
        }
        catch (\Exception $ex) {
          $msg = t('Tried to delete Job in Content API, but failed: @message', ['message' => $ex->getMessage()]);
          if (strlen($msg) > 200) {
            $msg = substr($msg, 0, 200);
          }
          \Drupal::messenger()->addMessage($msg, "error");
        }
        // TODO: Check why zip cannot be deleted. But after two tests seems to work?
        $zipfileobj = GeneralHelper::createFileObject($zipPath);
        \Drupal::service('file_system')->delete($zipfileobj->getFileUri());
        \Drupal::service('file_system')->deleteRecursive($allfilespath);

        $msg = $exception->getMessage();
        if (strlen($msg) > 200) {
          $msg = substr($msg, 0, 200);
        }
        \Drupal::messenger()->addMessage($msg, 'error');
        // \Drupal::messenger()->addMessage($exception->getMessage(), "error");
      }
      return $createdcpjob;
    }

    /**
     * Request translation reference.
     */
    private function requestTranslationRef(JobInterface $job, $createdcpjob) {
      // Defined here as they will be used in catch block.
      $token = NULL;
      $zipPath = NULL;
      $ziparchive = NULL;
      $jobapi = NULL;
      $allfilespath = NULL;
      try {
        // Message which will be displayed using drupa_set_message to disaplay download.
        $messageTopass = 'Exported files can be downloaded here: <br/>';
        $translator = $job->getTranslator();
        $contentapisettings = $translator->getSetting("capi-settings");
        $oneexportfile = $translator->getSetting("one_export_file");
        $capisettings = $translator->getSetting('capi-settings');
        $capi = new TokenApi();
        $token = $capi->getToken($capisettings['capi_username_ctt'], $capisettings['capi_password_ctt']);
        if ($job->getSetting('capi-settings')["task"] == "trans") {
          // Export files: gernerate paths and other variables.
          $exporter = new Xliff();
          $filesystem = \Drupal::service('file_system');
          $joblabel = GeneralHelper::getJobLabelNoSpeChars($job);

          $dirnameallfiles = $joblabel . '_' . $job->id() . "_" . $job->getRemoteSourceLanguage() . "_" . $job->getRemoteTargetLanguage();
          $zipName = 'zip_ref_' . $dirnameallfiles . '.zip';
          $allfilespath = $job->getSetting('scheme') . '://tmgmt_contentapi/LioxRefFiles/' . $dirnameallfiles;
          $zipPath = $allfilespath . "/" . $zipName;
          $filearraytodelte = [];
          $filearrayexportedfiles = [];
          $transferfiles = [];
          // Create folder where all exported files will be stored.

          // Prepare reference files which needs to transfer with job.
          $transferfiles = \Drupal::service('tmgmt_contentapi.export_job')->refFilesToTransfer($job);

          // Create Job in Content API.
          $contentapibundle = [];
          $jobapi = new JobApi();
          // Upload files from transferfiles array to CPA.
          $fileapi = new SourceFileApi();
          $transrequestapi = new RequestApi();
          // Array to store temporally translation requst and associated files.
          $contentapitrrequstfiles = [];
          foreach ($transferfiles as $id => $tmpfile) {
            if ($tmpfile == NULL) {
              continue;
            }
            $data = [];
            $data["job_id"] = $createdcpjob->getJobId();

            $data["filename"] = $tmpfile->getFilename();
            $data["filetype"] = $tmpfile->getMimeType();
            $stmrg = \Drupal::service('stream_wrapper_manager')->getViaUri($tmpfile->getFileUri());
            $extpath = $stmrg->realpath();
            if ($extpath == NULL) {
              continue;
            }
            $filrequst = new \SplFileObject($extpath);

            // File upload to TUS CLINET.

            // STARTS UPLOAD SOURCE.
            $name = $data["filename"];

            $contentapitmpfile = $fileapi->jobsJobIdUploadPost($token, $data["job_id"], $data["filename"], $data["filetype"], $filrequst);
            $uploadedFile = $fileapi->uploadFile($contentapitmpfile, /* $jobpath*/$extpath);
            $request = new CreateRequestFile([
              "request_name" => $job->id() . "_" . $id . "_" . $job->getRemoteSourceLanguage() . "-" . $job->getRemoteTargetLanguage(),
              "source_native_id" => $job->id() . "_" . $id,
              "source_native_language_code" => $job->getRemoteSourceLanguage(),
              "target_native_language_codes" => [$job->getRemoteTargetLanguage()],
              "file_id" => $contentapitmpfile['fms_file_id'],
            ]);
            $contentapitmprequst = $transrequestapi->jobsJobIdRequestsAddfilePost($token, $data["job_id"], $request, 2);
            $contentapibundle[] = $contentapitmprequst;
            $filrequst = NULL;

          }

          // Delete files after sending to Provider.
          foreach ($filearraytodelte as $tempfile) {
            \Drupal::service('file_system')->delete($tempfile->getFileUri());
          }

        }
        else {
          // Export files: gernerate paths and other variables.
          $exporter = new Xliff();
          $filesystem = \Drupal::service('file_system');
          $joblabel = GeneralHelper::getJobLabelNoSpeChars($job);
          $dirnameallfiles = $joblabel . '_tmupdate' . $job->id() . "_" . $job->getRemoteSourceLanguage() . "_" . $job->getRemoteTargetLanguage();
          $zipName = 'zip_ref_' . $dirnameallfiles . '.zip';
          $allfilespath = $job->getSetting('scheme') . '://tmgmt_contentapi/LioxRefFiles/' . $dirnameallfiles;
          $zipPath = $allfilespath . "/" . $zipName;
          $filearraytodelte = [];
          $filearrayexportedfiles = [];
          $transferfiles = [];
          // Create folder where all exported files will be stored.
          if (\Drupal::service('file_system')->prepareDirectory($allfilespath, FileSystemInterface::CREATE_DIRECTORY)) {
            // Export each item of the job in same file.
            if ($oneexportfile) {
              $labelname = $joblabel;
              $name = $labelname . "_" . $job->id() . "_all_" . $job->getRemoteSourceLanguage() . '_' . $job->getRemoteTargetLanguage() . '.xlf';
              $jobpath = $allfilespath . "/" . $name;
              $file = \Drupal::service('file.repository')->writeData($exporter->export($job), $jobpath, FileSystemInterface::EXISTS_REPLACE);
              $filearraytodelte[] = $file;
              $filearrayexportedfiles['all'] = $file;
            }
            else {
              // Export each item of the job in separate file.
              foreach ($job->getItems() as $item) {
                $labelname = GeneralHelper::getStringNoSpeChars($item->label());
                $name = $labelname . "_" . $job->id() . "_" . $item->id() . "_" . $job->getRemoteSourceLanguage() . '_' . $job->getRemoteTargetLanguage() . '.xlf';
                $itempath = $allfilespath . "/" . $name;
                $file = \Drupal::service('file.repository')->writeData($exporter->exportItem($item), $itempath, FileSystemInterface::EXISTS_REPLACE);
                $filearraytodelte[] = $file;
                $filearrayexportedfiles[$item->id()] = $file;
              }
            }
            // Zip the exported files.
            $ziparchive = new \ZipArchive();
            $openresult = $ziparchive->open($filesystem->realpath($zipPath), ZIPARCHIVE::CREATE | ZIPARCHIVE::OVERWRITE);
            $zipcloseresult = FALSE;
            $addedFiles = 0;
            if ($openresult) {
              foreach ($filearrayexportedfiles as $tempfile) {
                if ($tempfile == NULL || $tempfile->getFileUri() == NULL || strlen($tempfile->getFileUri()) == 0) {
                  continue;
                }
                $ziparchive->addFile($filesystem->realpath($tempfile->getFileUri()), $tempfile->getFilename());
                $addedFiles = $addedFiles + 1;
              }
              $zipcloseresult = FALSE;
              if ($addedFiles > 0) {
                $zipcloseresult = $ziparchive->close();
              }
              if ($zipcloseresult) {
                $zipfileobj = GeneralHelper::createFileObject($zipPath);
                \Drupal::service('file.usage')->add($zipfileobj, 'tmgmt_contentapi', 'tmgmt_job', $job->id());
              }
            }

            $transferfiles = $filearrayexportedfiles;

            // Create Job in Content API.
            $contentapibundle = [];
            $jobapi = new JobApi();
            // Upload files from transferfiles array to CPA.
            $fileapi = new SourceFileApi();
            $transrequestapi = new RequestApi();
            // Array to store temporally translation requst and associated files.
            $contentapitrrequstfiles = [];
            foreach ($transferfiles as $id => $tmpfile) {
              $data = [];
              $data["job_id"] = $createdcpjob->getJobId();
              $data["filename"] = $tmpfile->getFilename();
              $data["filetype"] = $tmpfile->getMimeType();
              $stmrg = \Drupal::service('stream_wrapper_manager')->getViaUri($tmpfile->getFileUri());
              $extpath = $stmrg->realpath();
              $filrequst = new \SplFileObject($extpath);

              $contentapitmpfile = $fileapi->jobsJobIdUploadPost($token, $data["job_id"], $data["filename"], $data["filetype"], $filrequst);
              $uploadedFile = $fileapi->uploadFile($contentapitmpfile, $extpath);

              $request = new CreateRequestFile([
                "request_name" => $job->id() . "_" . $id . "_" . $job->getRemoteSourceLanguage() . "-" . $job->getRemoteTargetLanguage(),
                "source_native_id" => $job->id() . "_" . $id,
                "source_native_language_code" => $job->getRemoteSourceLanguage(),
                "target_native_language_codes" => [$job->getRemoteTargetLanguage()],
                "file_id" => $contentapitmpfile['fms_file_id'],
              ]);
              $contentapitmprequst = $transrequestapi->jobsJobIdRequestsAddfilePost($token, $data["job_id"], $request, 2);
              $contentapibundle[] = $createdcpjob;
            }
            // Submit content api job.
            $pr = $job->getSetting("capi-settings")["provider"];
            $jid = $createdcpjob->getJobId();
            $prmodel = new ProviderId(['provider_id' => $pr]);
            $jobsubmitresult = $jobapi->jobsJobIdSubmitPut($token, $jid, $prmodel);
            if ($jobsubmitresult) {
              // TODO.
              GeneralHelper::addCpaSettingsToJob($job, serialize($contentapibundle));
              $messageTopass .= '<a href="' . \Drupal::service('file_url_generator')->generateAbsoluteString($zipPath) . '">' . Xss::filter(GeneralHelper::getJobLabelNoSpeChars($job)) . '</a>';
              \Drupal::messenger()->addMessage(Markup::create($messageTopass));
              // Set job to finished.
              $job->setState(Job::STATE_FINISHED);
              foreach ($job->getItems() as $item) {
                $item->setState(JobItem::STATE_ACCEPTED);

              }
              $job->submitted("TM update job sent to provider!");

              // Delete files after sending to Provider.
              foreach ($filearraytodelte as $tempfile) {
                \Drupal::service('file_system')->delete($tempfile->getFileUri());
              }
            }
          }
          else {
            throw new \Exception("Could not create directory for export: " . $allfilespath);
          }
        }
      }
      catch (\Exception $exception) {
        // If exception occurs, clean up everything: delete exported files, cancel job in CA if any.
        foreach ($filearraytodelte as $tempfile) {
          if (file_exists($tempfile->getFileUri())) {
            \Drupal::service('file_system')->delete($tempfile->getFileUri());
          }
        }
        if (!file_exists($zipPath) && $ziparchive != NULL) {
          $ziparchive->close();
        }
        $job->setState(Job::STATE_UNPROCESSED);
        try {
          if (isset($createdcpjob)) {
            // TODO: check why result is NULL, only at certain state job can be deleted? But seems to work!!
            $deletresult = $jobapi->jobsJobIdDelete($token, $createdcpjob->getJobId());
          }
        }
        catch (\Exception $ex) {
          $msg = t('Tried to delete Job in Content API, but failed: @message', ['message' => $ex->getMessage()]);
          if (strlen($msg) > 200) {
            $msg = substr($msg, 0, 200);
          }
          \Drupal::messenger()->addMessage($msg, "error");
        }
        $zipfileobj = GeneralHelper::createFileObject($zipPath);
        \Drupal::service('file_system')->delete($zipfileobj->getFileUri());
        \Drupal::service('file_system')->deleteRecursive($allfilespath);
        $msg = $exception->getMessage();
        if (strlen($msg) > 200) {
          $msg = substr($msg, 0, 200);
        }
        \Drupal::messenger()->addMessage($msg, 'error');
      }
    }

    /**
 * Keep this as back-up method in case we want to add all jobItems in one File.
 */
    #[\ReturnTypeWillChange]
    public function requestTranslationOrg(JobInterface $job) {
      $name = "JobID" . $job->id() . '_' . $job->getSourceLangcode() . '_' . $job->getTargetLangcode();
      $export = \Drupal::service('plugin.manager.tmgmt_fwconnector.format')->createInstance($job->getSetting('export_format'));
      $path = $job->getSetting('scheme') . '://tmgmt_fwconnector/' . $name . '.' . $job->getSetting('export_format');
      $dirname = dirname($path);

      if (\Drupal::service('file_system')->prepareDirectory($dirname, FileSystemInterface::CREATE_DIRECTORY)) {
        $file = \Drupal::service('file.repository')->writeData($export->export($job), $path);
        \Drupal::service('file.usage')->add($file, 'tmgmt_fwconnector', 'tmgmt_job', $job->id());
        $job->submitted('Exported file can be downloaded <a href="@link">here</a>.', ['@link' => \Drupal::service('file_url_generator')->generateAbsoluteString($path)]);
      }
    }

    /**
     * {@inheritdoc}
     */
    #[\ReturnTypeWillChange]
    public function hasCheckoutSettings(JobInterface $job) {
      return $job->getTranslator()->getSetting('allow_override');
    }

    /**
     * {@inheritdoc}
     */
    #[\ReturnTypeWillChange]
    public function defaultSettings() {
      return [
      'export_format' => 'contentapi_xlf',
      'allow_override' => TRUE,
      'scheme' => 'public',
      // Making this setting TRUE by default is more appropriate, however we
      // need to make it FALSE due to backwards compatibility.
      'xliff_processing' => FALSE,
      'xliff_cdata' => FALSE,
      'capi_username_ctt' => '',
      'one_export_file' => TRUE,
      'po_reference' => '',
      'transfer-settings' => FALSE
    ];
    }

    /**
     * {@inheritdoc}
     */
    #[\ReturnTypeWillChange]
    public function requestJobItemsTranslation(array $job_items) {
      // TODO: Implement requestJobItemsTranslation() method.

    }

}
